import { AdRepositoryPort } from '@modules/ad/core/application/ports/ad.repository.port';
import { MatchQuery } from '@modules/ad/core/application/queries/match/match.query';
import { PassengerOrientedAlgorithm } from '@modules/ad/core/application/queries/match/passenger-oriented-algorithm';
import { AlgorithmType } from '@modules/ad/core/application/types/algorithm.types';
import { Waypoint } from '@modules/ad/core/application/types/waypoint.type';
import { Frequency } from '@modules/ad/core/domain/ad.types';
import { MatchEntity } from '@modules/ad/core/domain/match.entity';

const originWaypoint: Waypoint = {
  position: 0,
  lat: 48.689445,
  lon: 6.17651,
  houseNumber: '5',
  street: 'Avenue Foch',
  locality: 'Nancy',
  postalCode: '54000',
  country: 'France',
};
const destinationWaypoint: Waypoint = {
  position: 1,
  lat: 48.8566,
  lon: 2.3522,
  locality: 'Paris',
  postalCode: '75000',
  country: 'France',
};

const matchQuery = new MatchQuery(
  {
    algorithmType: AlgorithmType.PASSENGER_ORIENTED,
    driver: false,
    passenger: true,
    frequency: Frequency.PUNCTUAL,
    fromDate: '2023-08-28',
    toDate: '2023-08-28',
    schedule: [
      {
        time: '07:05',
      },
    ],
    strict: false,
    waypoints: [originWaypoint, destinationWaypoint],
  },
  {
    getBasic: jest.fn().mockImplementation(() => ({
      duration: 6500,
      distance: 89745,
    })),
    getDetailed: jest.fn(),
  },
);

const mockMatcherRepository: AdRepositoryPort = {
  insertExtra: jest.fn(),
  findOneById: jest.fn(),
  findOne: jest.fn(),
  findAll: jest.fn(),
  findAllByIds: jest.fn(),
  insert: jest.fn(),
  update: jest.fn(),
  updateWhere: jest.fn(),
  delete: jest.fn(),
  count: jest.fn(),
  healthCheck: jest.fn(),
  getCandidateAds: jest.fn().mockImplementation(() => [
    {
      id: 'cc260669-1c6d-441f-80a5-19cd59afb777',
      getProps: jest.fn().mockImplementation(() => ({
        waypoints: [
          {
            lat: 48.6645,
            lon: 6.18457,
          },
          {
            lat: 48.7898,
            lon: 2.36845,
          },
        ],
      })),
    },
  ]),
};

describe('Passenger oriented algorithm', () => {
  it('should return matching entities', async () => {
    const passengerOrientedAlgorithm: PassengerOrientedAlgorithm =
      new PassengerOrientedAlgorithm(matchQuery, mockMatcherRepository);
    const matches: MatchEntity[] = await passengerOrientedAlgorithm.match();
    expect(matches.length).toBeGreaterThanOrEqual(0);
  });
});
